---------------------------------------------------------------------------------
-- Company:        CERN, BE-CO-HT 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    18:15:56 11/09/2011 
-- Design Name:    A I2C slave with wishbone slave output and interrupt for MCU
-- Module Name:    i2c_slave_top - Behavioral 
-- Project Name:   CTDAH 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity i2c_slave_top is
   port (
      sda_oen    : out STD_LOGIC;
      sda_i     : in  STD_LOGIC;
      sda_o     : out STD_LOGIC;
      scl_oen    : out STD_LOGIC;
      scl_i     : in  STD_LOGIC;
      scl_o     : out STD_LOGIC;
      wb_clk    : in  STD_LOGIC;
      wb_rst_i  : in  STD_LOGIC;
      wb_stb_i  : in  STD_LOGIC;
      wb_cyc_i  : in  STD_LOGIC;
      wb_sel_i  : in  STD_LOGIC;
      wb_we_i   : in  STD_LOGIC;
      wb_data_i : in  STD_LOGIC_VECTOR (31 downto 0);
      wb_data_o : out STD_LOGIC_VECTOR (31 downto 0);
      wb_addr_i : in  STD_LOGIC_VECTOR (3 downto 0);
      wb_ack_o  : out STD_LOGIC;
      wb_rty_o  : out STD_LOGIC;
      wb_err_o  : out STD_LOGIC;
      ind_wb_addr  : out  STD_LOGIC;
      inst_rd      : out  STD_LOGIC;
      inst_wr      : out  STD_LOGIC 
   );
end i2c_slave_top;

architecture Behavioral of i2c_slave_top is


signal PRE_s     : STD_LOGIC_VECTOR (15 downto 0);
signal CTR0_s    : STD_LOGIC_VECTOR (15  downto 0);
signal CTR1_s    : STD_LOGIC_VECTOR (15  downto 0);
signal STA_s     : STD_LOGIC_VECTOR (15 downto 0);
signal DRX0_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DRX1_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DRX2_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DRX3_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DRX4_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DRX5_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DTX0_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DTX1_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DTX2_s    : STD_LOGIC_VECTOR (7  downto 0);
signal DTX3_s    : STD_LOGIC_VECTOR (7  downto 0);
signal load_TX_s      : STD_LOGIC;
signal tx_fifo_oen_s  : STD_LOGIC;

   component i2c_slave_core
   port (
      wb_clk    : in  STD_LOGIC;
      wb_rst_i  : in  STD_LOGIC;
      sda_oen    : out  STD_LOGIC;
      sda_i     : in   STD_LOGIC;
      sda_o     : out  STD_LOGIC;
      scl_oen    : out  STD_LOGIC;
      scl_i     : in   STD_LOGIC;
      scl_o     : out  STD_LOGIC;
      PRE_i     : in   STD_LOGIC_VECTOR (15 downto 0);
      CTR0_i    : in   STD_LOGIC_VECTOR (15  downto 0);
      CTR1_i    : in   STD_LOGIC_VECTOR (15  downto 0);
      STA_o     : out  STD_LOGIC_VECTOR (15 downto 0);
      DRX0_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DRX1_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DRX2_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DRX3_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DRX4_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DRX5_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DTX0_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DTX1_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DTX2_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DTX3_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      load_TX      : in   STD_LOGIC;
      tx_fifo_oen  : in   STD_LOGIC;
      ind_wb_addr  : out  STD_LOGIC;
      inst_rd      : out  STD_LOGIC;
      inst_wr      : out  STD_LOGIC 
   );
   end component;

   component i2c_regs
   port (
      -- These are the registers offers to others modules of the FPGA
      wb_rst_i  : in  STD_LOGIC;
      wb_clk    : in  STD_LOGIC;
      wb_we_i   : in  STD_LOGIC;
      wb_stb_i  : in  STD_LOGIC;
      wb_cyc_i  : in  STD_LOGIC;
      wb_sel_i  : in  STD_LOGIC;
      wb_data_i : in  STD_LOGIC_VECTOR (31 downto 0);
      wb_data_o : out STD_LOGIC_VECTOR (31 downto 0);
      wb_addr_i : in  STD_LOGIC_VECTOR (3  downto 0);
      wb_ack_o  : out STD_LOGIC;
      wb_rty_o  : out STD_LOGIC;
      wb_err_o  : out STD_LOGIC;

      i2c_addr  : in  STD_LOGIC_VECTOR (6  downto 0);
      -- These are the registers that are offered to the i2c slave core
      STA_i     : in   STD_LOGIC_VECTOR (15 downto 0);
      PRE_o     : out  STD_LOGIC_VECTOR (15 downto 0);
      CTR0_o    : out  STD_LOGIC_VECTOR (15 downto 0);
      CTR1_o    : out  STD_LOGIC_VECTOR (15 downto 0);
      DRX0_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX1_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX2_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX3_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX4_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DRX5_i    : in   STD_LOGIC_VECTOR (7  downto 0);
      DTX0_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DTX1_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DTX2_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      DTX3_o    : out  STD_LOGIC_VECTOR (7  downto 0);
      load_TX     : out  STD_LOGIC;
      tx_fifo_oen : out  STD_LOGIC
   );
   end component;


begin

   inst_i2c_slave_core: i2c_slave_core
   port map(
       wb_clk    => wb_clk,
       wb_rst_i  => wb_rst_i,
       sda_oen    => sda_oen,
       sda_i     => sda_i,
       sda_o     => sda_o,
       scl_oen    => scl_oen,
       scl_i     => scl_i,
       scl_o     => scl_o,
       PRE_i     => PRE_s,
       CTR0_i    => CTR0_s,
       CTR1_i    => CTR1_s,
       STA_o     => STA_s,
       DRX0_o    => DRX0_s,
       DRX1_o    => DRX1_s,
       DRX2_o    => DRX2_s,
       DRX3_o    => DRX3_s,
       DRX4_o    => DRX4_s,
       DRX5_o    => DRX5_s,
       DTX0_i    => DTX0_s,
       DTX1_i    => DTX1_s,
       DTX2_i    => DTX2_s,
       DTX3_i    => DTX3_s,
       load_tx   => load_tx_s,
       tx_fifo_oen  => tx_fifo_oen_s,
       ind_wb_addr  => ind_wb_addr,
       inst_rd      => inst_rd,
       inst_wr      => inst_wr 
   );

   inst_i2c_regs: i2c_regs
   port map(
       wb_rst_i  => wb_rst_i,
       wb_clk    => wb_clk,
       wb_we_i   => wb_we_i,
       wb_stb_i  => wb_stb_i,
       wb_cyc_i  => wb_cyc_i,
       wb_sel_i  => wb_sel_i,
       wb_data_i => wb_data_i,
       wb_data_o => wb_data_o,
       wb_addr_i => wb_addr_i,
       wb_ack_o  => wb_ack_o,
       wb_rty_o  => wb_rty_o,
       wb_err_o  => wb_err_o,
       i2c_addr  => "0001111",
       PRE_o     => PRE_s,  
       CTR0_o    => CTR0_s,  
       CTR1_o    => CTR1_s,  
       STA_i     => STA_s,  
       DRX0_i    => DRX0_s,  
       DRX1_i    => DRX1_s,  
       DRX2_i    => DRX2_s,  
       DRX3_i    => DRX3_s,  
       DRX4_i    => DRX4_s,  
       DRX5_i    => DRX5_s,  
       DTX0_o    => DTX0_s,  
       DTX1_o    => DTX1_s,  
       DTX2_o    => DTX2_s,  
       DTX3_o    => DTX3_s,
       load_tx   => load_tx_s,
       tx_fifo_oen  => tx_fifo_oen_s
   );



end Behavioral;

