library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

use work.gencores_pkg.all;
use work.wrcore_pkg.all;
use work.wr_fabric_pkg.all;

library UNISIM;
use UNISIM.vcomponents.all;

library work;
use work.wishbone_pkg.all;


entity wr_ep is
   generic
   (
      TAR_ADDR_WDTH : integer := 13        -- not used for this project
   );
   port
   (
      -- 1 rst_i
      -- 2 button1_i      -- What for?
      -- 3 button2_i      -- What for?
      EXTRA_SWITCH    : in  STD_LOGIC_VECTOR(3 downto 1);

      -------------------------------------------------------------------------
      -- Clocks
      -------------------------------------------------------------------------
      CLK20_VCXO         : in  STD_LOGIC;
      FPGA_CLK_P         : in  STD_LOGIC;  --clk_125m_pllref_p_i
      FPGA_CLK_N         : in  STD_LOGIC;  --clk_125m_pllref_n_i

      FPGAMGTCLK0_P      : in STD_LOGIC; --fpga_pll_ref_clk_101_p_i   -- Dedicated clock for Xilinx GTP transceiver
      FPGAMGTCLK0_N      : in STD_LOGIC; --fpga_pll_ref_clk_101_n_i

      -------------------------------------------------------------------------
      -- Font panel LEDs
      -------------------------------------------------------------------------
      LED_WR_OK                  : out STD_LOGIC;  --led_red
      LED_PPS                    : out STD_LOGIC;  --led_green

      -------------------------------------------------------------------------
      -- 1-Wire interface to DS18B20
      -------------------------------------------------------------------------
      THERMOMETER           : inout STD_LOGIC;     --thermo_id

      -------------------------------------------------------------------------
      -- DAC control
      -------------------------------------------------------------------------
      FPGA_PLLDAC1_SYNC_N   : out STD_LOGIC;      -- dac_cs1_n_o 
      FPGA_PLLDAC_SCLK      : out STD_LOGIC;      -- dac_sclk1_o 
      FPGA_PLLDAC_DIN       : out STD_LOGIC;      -- dac_din1_o  
--      dac_clr1_n_o          : out STD_LOGIC;      -- dac_clr1_n_o

      FPGA_PLLDAC2_SYNC_N   : out STD_LOGIC;      -- dac_cs2_n_o 
--      dac_sclk2_o         : out STD_LOGIC;      -- dac_sclk2_o 
--      dac_din2_o          : out STD_LOGIC;      -- dac_din2_o  
--      dac_clr2_n_o        : out STD_LOGIC;      -- dac_clr2_n_o

      -------------------------------------------------------------------------
      -- SFP pins
      -------------------------------------------------------------------------
      MGTSFPTX0_P           : out STD_LOGIC;      --sfp_txp_o
      MGTSFPTX0_N           : out STD_LOGIC;      --sfp_txn_o
      MGTSFPRX0_P           : in  STD_LOGIC;      --sfp_rxp_i
      MGTSFPRX0_N           : in  STD_LOGIC;      --sfp_rxn_i

      FPGA_SFP_PRESENCE     : in    STD_LOGIC;    --sfp_mod_def0_b     --sfp detect
      FPGA_SFP_SCL          : inout STD_LOGIC;    --sfp_mod_def1_b     --scl
      FPGA_SFP_SDA          : inout STD_LOGIC;    --sfp_mod_def2_b     --sda
      FPGA_SFP_RATE_SELECT  : inout STD_LOGIC;    --sfp_rate_select_b
      SFP_TX_FAULT          : in    STD_LOGIC;
      SFP_TX_DISABLE        : out   STD_LOGIC;    --sfp_tx_disable_o
      FPGA_SFP_LOS          : in    STD_LOGIC
   );

end wr_ep;

architecture rtl of wr_ep is

   ------------------------------------------------------------------------------
   -- Components declaration
   ------------------------------------------------------------------------------

   component xwr_core is
      generic
      (
         g_SIMULATION                : INTEGER                        := 0;
         g_PHYS_UART                 : BOOLEAN                        := TRUE;
         g_VIRTUAL_UART              : BOOLEAN                        := FALSE;
         g_WITH_EXTERNAL_CLOCK_INPUT : BOOLEAN                        := FALSE;
         g_AUX_CLKS                  : INTEGER                        := 1;
         g_EP_RXBUF_SIZE             : INTEGER                        := 1024;
         g_DPRAM_INITF               : STRING                         := "";
         g_DPRAM_SIZE                : INTEGER                        := 20480;  --in 32-bit words
         g_INTERFACE_MODE            : t_wishbone_interface_mode      := CLASSIC;
         g_ADDRESS_GRANULARITY       : t_wishbone_address_granularity := WORD
      );
      port
      (
         clk_sys_i            : in  STD_LOGIC;
         clk_dmtd_i           : in  STD_LOGIC;
         clk_ref_i            : in  STD_LOGIC;
         clk_aux_i            : in  STD_LOGIC_VECTOR(g_AUX_CLKS-1 downto 0) := (others => '0');
         clk_ext_i            : in  STD_LOGIC := '0';
         pps_ext_i            : in  STD_LOGIC := '0';
         rst_n_i              : in  STD_LOGIC;

         dac_hpll_load_p1_o   : out STD_LOGIC;
         dac_hpll_data_o      : out STD_LOGIC_VECTOR(15 downto 0);
         dac_dpll_load_p1_o   : out STD_LOGIC;
         dac_dpll_data_o      : out STD_LOGIC_VECTOR(15 downto 0);

         phy_ref_clk_i        : in  STD_LOGIC;
         phy_tx_data_o        : out STD_LOGIC_VECTOR(7 downto 0);
         phy_tx_k_o           : out STD_LOGIC;
         phy_tx_disparity_i   : in  STD_LOGIC;
         phy_tx_enc_err_i     : in  STD_LOGIC;
         phy_rx_data_i        : in  STD_LOGIC_VECTOR(7 downto 0);
         phy_rx_rbclk_i       : in  STD_LOGIC;
         phy_rx_k_i           : in  STD_LOGIC;
         phy_rx_enc_err_i     : in  STD_LOGIC;
         phy_rx_bitslide_i    : in  STD_LOGIC_VECTOR(3 downto 0);
         phy_rst_o            : out STD_LOGIC;
         phy_loopen_o         : out STD_LOGIC;

         led_red_o            : out STD_LOGIC;
         led_green_o          : out STD_LOGIC;
         scl_o                : out STD_LOGIC;
         scl_i                : in  STD_LOGIC;
         sda_o                : out STD_LOGIC;
         sda_i                : in  STD_LOGIC;
         sfp_scl_o            : out STD_LOGIC;
         sfp_scl_i            : in  STD_LOGIC;
         sfp_sda_o            : out STD_LOGIC;
         sfp_sda_i            : in  STD_LOGIC;
         sfp_det_i            : in  STD_LOGIC;
         btn1_i               : in  STD_LOGIC;
         btn2_i               : in  STD_LOGIC;

         uart_rxd_i           : in  STD_LOGIC;
         uart_txd_o           : out STD_LOGIC;

         owr_en_o             : out STD_LOGIC_VECTOR(1 downto 0);
         owr_i                : in  STD_LOGIC_VECTOR(1 downto 0);

         slave_i              : in  t_wishbone_slave_in;
         slave_o              : out t_wishbone_slave_out;

         wrf_src_o            : out t_wrf_source_out;
         wrf_src_i            : in  t_wrf_source_in := c_dummy_src_in;
         wrf_snk_o            : out t_wrf_sink_out;
         wrf_snk_i            : in  t_wrf_sink_in   := c_dummy_snk_in;

         timestamps_o         : out t_txtsu_timestamp;
         timestamps_ack_i     : in  std_logic := '1';

         tm_dac_value_o       : out std_logic_vector(23 downto 0);
         tm_dac_wr_o          : out std_logic;
         tm_clk_aux_lock_en_i : in  std_logic;
         tm_clk_aux_locked_o  : out std_logic;
         tm_time_valid_o      : out std_logic;
         tm_utc_o             : out std_logic_vector(39 downto 0);
         tm_cycles_o          : out std_logic_vector(27 downto 0);
         pps_p_o              : out std_logic;

         dio_o                : out std_logic_vector(3 downto 0);
         rst_aux_n_o          : out std_logic
      );
   end component;

   component wr_gtp_phy_spartan6
      generic
      (
      g_simulation         : INTEGER;
      g_ch0_use_refclk_out : BOOLEAN := FALSE;
      g_ch1_use_refclk_out : BOOLEAN := FALSE
   );
   port (
           gtp_clk_i          : in  STD_LOGIC;
           ch0_ref_clk_i      : in  STD_LOGIC;
           ch0_tx_data_i      : in  STD_LOGIC_VECTOR(7 downto 0);
           ch0_tx_k_i         : in  STD_LOGIC;
           ch0_tx_disparity_o : out STD_LOGIC;
           ch0_tx_enc_err_o   : out STD_LOGIC;
           ch0_rx_rbclk_o     : out STD_LOGIC;
           ch0_rx_data_o      : out STD_LOGIC_VECTOR(7 downto 0);
           ch0_rx_k_o         : out STD_LOGIC;
           ch0_rx_enc_err_o   : out STD_LOGIC;
           ch0_rx_bitslide_o  : out STD_LOGIC_VECTOR(3 downto 0);
           ch0_rst_i          : in  STD_LOGIC;
           ch0_loopen_i       : in  STD_LOGIC;
           ch1_ref_clk_i      : in  STD_LOGIC;
           ch1_tx_data_i      : in  STD_LOGIC_VECTOR(7 downto 0) := "00000000";
           ch1_tx_k_i         : in  STD_LOGIC                    := '0';
           ch1_tx_disparity_o : out STD_LOGIC;
           ch1_tx_enc_err_o   : out STD_LOGIC;
           ch1_rx_data_o      : out STD_LOGIC_VECTOR(7 downto 0);
           ch1_rx_rbclk_o     : out STD_LOGIC;
           ch1_rx_k_o         : out STD_LOGIC;
           ch1_rx_enc_err_o   : out STD_LOGIC;
           ch1_rx_bitslide_o  : out STD_LOGIC_VECTOR(3 downto 0);
           ch1_rst_i          : in  STD_LOGIC                    := '0';
           ch1_loopen_i       : in  STD_LOGIC                    := '0';
           pad_txn0_o         : out STD_LOGIC;
           pad_txp0_o         : out STD_LOGIC;
           pad_rxn0_i         : in  STD_LOGIC                    := '0';
           pad_rxp0_i         : in  STD_LOGIC                    := '0';
           pad_txn1_o         : out STD_LOGIC;
           pad_txp1_o         : out STD_LOGIC;
           pad_rxn1_i         : in  STD_LOGIC                    := '0';
           pad_rxp1_i         : in  STD_LOGIC                    := '0');
   end component;

   component spec_serial_dac_arb
      generic
      (
      g_invert_sclk    : BOOLEAN;
      g_num_extra_bits : INTEGER
   );
   port
   (
      clk_i       : in  STD_LOGIC;
      rst_n_i     : in  STD_LOGIC;
      val1_i      : in  STD_LOGIC_VECTOR(15 downto 0);
      load1_i     : in  STD_LOGIC;
      val2_i      : in  STD_LOGIC_VECTOR(15 downto 0);
      load2_i     : in  STD_LOGIC;
      dac_cs_n_o  : out STD_LOGIC_VECTOR(1 downto 0);
      dac_clr_n_o : out STD_LOGIC;
      dac_sclk_o  : out STD_LOGIC;
      dac_din_o   : out STD_LOGIC);
   end component;

   ------------------------------------------------------------------------------
   -- Constants declaration
   ------------------------------------------------------------------------------
   constant c_BAR0_APERTURE     : INTEGER := 20;
   constant c_CSR_WB_SLAVES_NB  : INTEGER := 1;
   constant c_DMA_WB_SLAVES_NB  : INTEGER := 1;
   constant c_DMA_WB_ADDR_WIDTH : INTEGER := 26;

   ------------------------------------------------------------------------------
   -- Signals declaration
   ------------------------------------------------------------------------------

   signal gtp_dedicated_clk : STD_LOGIC;

   -- P2L colck PLL status
   signal p2l_pll_locked : STD_LOGIC;

   -- Reset
   signal s_rst_a : STD_LOGIC;
   signal s_rst   : STD_LOGIC;

   -- DMA wishbone bus
   --signal dma_adr     : std_logic_vector(31 downto 0);
   --signal dma_dat_i   : std_logic_vector((32*c_DMA_WB_SLAVES_NB)-1 downto 0);
   --signal dma_dat_o   : std_logic_vector(31 downto 0);
   --signal dma_sel     : std_logic_vector(3 downto 0);
   --signal dma_cyc     : std_logic;  --_vector(c_DMA_WB_SLAVES_NB-1 downto 0);
   --signal dma_stb     : std_logic;
   --signal dma_we      : std_logic;
   --signal dma_ack     : std_logic;  --_vector(c_DMA_WB_SLAVES_NB-1 downto 0);
   --signal dma_stall   : std_logic;  --_vector(c_DMA_WB_SLAVES_NB-1 downto 0);
   signal ram_we      : STD_LOGIC_VECTOR(0 downto 0);
   signal ddr_dma_adr : STD_LOGIC_VECTOR(29 downto 0);

   signal irq_to_gn4124 : STD_LOGIC;

   -- SPI
   signal spi_slave_select : STD_LOGIC_VECTOR(7 downto 0);


   signal pllout_clk_sys       : STD_LOGIC;
   signal pllout_clk_dmtd      : STD_LOGIC;
   signal pllout_clk_fb_pllref : STD_LOGIC;
   signal pllout_clk_fb_dmtd   : STD_LOGIC;

   signal clk_20m_vcxo_buf : STD_LOGIC;
   signal s_fpga_clk       : STD_LOGIC;
   signal clk_sys          : STD_LOGIC;
   signal clk_dmtd         : STD_LOGIC;
   signal dac_rst_n        : STD_LOGIC;
   signal led_divider      : UNSIGNED(23 downto 0);

   signal wrc_scl_o : STD_LOGIC;
   signal wrc_scl_i : STD_LOGIC;
   signal wrc_sda_o : STD_LOGIC;
   signal wrc_sda_i : STD_LOGIC;
   signal sfp_scl_o : STD_LOGIC;
   signal sfp_scl_i : STD_LOGIC;
   signal sfp_sda_o : STD_LOGIC;
   signal sfp_sda_i : STD_LOGIC;
   signal dio       : STD_LOGIC_VECTOR(3 downto 0);

   signal dac_hpll_load_p1 : STD_LOGIc;
   signal dac_dpll_load_p1 : STD_LOGIc;
   signal dac_hpll_data    : STD_LOGIC_VECTOR(15 downto 0);
   signal dac_dpll_data    : STD_LOGIC_VECTOR(15 downto 0);

   signal pps : STD_LOGIc;

   signal phy_tx_data      : STD_LOGIC_VECTOR(7 downto 0);
   signal phy_tx_k         : STD_LOGIC;
   signal phy_tx_disparity : STD_LOGIC;
   signal phy_tx_enc_err   : STD_LOGIC;
   signal phy_rx_data      : STD_LOGIC_VECTOR(7 downto 0);
   signal phy_rx_rbclk     : STD_LOGIC;
   signal phy_rx_k         : STD_LOGIC;
   signal phy_rx_enc_err   : STD_LOGIC;
   signal phy_rx_bitslide  : STD_LOGIC_VECTOR(3 downto 0);
   signal phy_rst          : STD_LOGIc;
   signal phy_loopen       : STD_LOGIc;

   signal dio_in         : STD_LOGIC_VECTOr(4 downto 0);
   signal dio_out        : STD_LOGIC_VECTOr(4 downto 0);
   signal dio_clk        : STD_LOGIC;

   signal local_reset_n  : STD_LOGIC;
   signal button1_synced : STD_LOGIC_VECTOR(2 downto 0);

   signal wrc_slave_i    : t_wishbone_slave_in;
   signal wrc_slave_o    : t_wishbone_slave_out;

   signal owr_en         : STD_LOGIC_VECTOR(1 downto 0);
   signal owr_i          : STD_LOGIC_VECTOR(1 downto 0);

   signal wb_adr         : STD_LOGIC_VECTOR(31 downto 0);  --c_BAR0_APERTURE-priv_log2_ceil(c_CSR_WB_SLAVES_NB+1)-1 downto 0);

   signal s_dac_sclk_o   : STD_LOGIC;
   signal s_dac_din_o   : STD_LOGIC;

   component xmini_bone
      generic (
      g_class_mask    : STD_LOGIC_VECTOR(7 downto 0);
      g_our_ethertype : STD_LOGIC_VECTOR(15 downto 0));
      port (
              clk_sys_i : in  STD_LOGIc;
              rst_n_i   : in  STD_LOGIc;
              src_o     : out t_wrf_source_out;
              src_i     : in  t_wrf_source_in;
              snk_o     : out t_wrf_sink_out;
              snk_i     : in  t_wrf_sink_in;
              master_o  : out t_wishbone_master_out;
              master_i  : in  t_wishbone_master_in);
   end component;

   signal mbone_rst_n     : STD_LOGIC;
   signal mbone_src_out   : t_wrf_source_out;
   signal mbone_src_in    : t_wrf_source_in;
   signal mbone_snk_out   : t_wrf_sink_out;
   signal mbone_snk_in    : t_wrf_sink_in;
   signal mbone_wb_out    : t_wishbone_master_out;
   signal mbone_wb_in     : t_wishbone_master_in;
   signal dpram_slave2_in : t_wishbone_master_out;

begin

   cmp_sys_clk_pll : PLL_BASE
   generic map
   (
      BANDWIDTH          => "OPTIMIZED",
      CLK_FEEDBACK       => "CLKFBOUT",
      COMPENSATION       => "INTERNAL",
      DIVCLK_DIVIDE      => 1,
      CLKFBOUT_MULT      => 8,
      CLKFBOUT_PHASE     => 0.000,
      CLKOUT0_DIVIDE     => 16,         -- 62.5 MHz
      CLKOUT0_PHASE      => 0.000,
      CLKOUT0_DUTY_CYCLE => 0.500,
      CLKOUT1_DIVIDE     => 16,         -- 125 MHz
      CLKOUT1_PHASE      => 0.000,
      CLKOUT1_DUTY_CYCLE => 0.500,
      CLKOUT2_DIVIDE     => 16,
      CLKOUT2_PHASE      => 0.000,
      CLKOUT2_DUTY_CYCLE => 0.500,
      CLKIN_PERIOD       => 8.0,
      REF_JITTER         => 0.016
   )
   port map
   (
      CLKFBOUT => pllout_clk_fb_pllref,
      CLKOUT0  => pllout_clk_sys,
      CLKOUT1  => open,
      CLKOUT2  => open,
      CLKOUT3  => open,
      CLKOUT4  => open,
      CLKOUT5  => open,
      LOCKED   => open,
      RST      => '0',
      CLKFBIN  => pllout_clk_fb_pllref,
      CLKIN    => s_fpga_clk);

   cmp_dmtd_clk_pll : PLL_BASE
   generic map
   (
      BANDWIDTH          => "OPTIMIZED",
      CLK_FEEDBACK       => "CLKFBOUT",
      COMPENSATION       => "INTERNAL",
      DIVCLK_DIVIDE      => 1,
      CLKFBOUT_MULT      => 50,
      CLKFBOUT_PHASE     => 0.000,
      CLKOUT0_DIVIDE     => 8,          -- 62.5 MHz
      CLKOUT0_PHASE      => 0.000,
      CLKOUT0_DUTY_CYCLE => 0.500,
      CLKOUT1_DIVIDE     => 8,          -- 125 MHz
      CLKOUT1_PHASE      => 0.000,
      CLKOUT1_DUTY_CYCLE => 0.500,
      CLKOUT2_DIVIDE     => 8,
      CLKOUT2_PHASE      => 0.000,
      CLKOUT2_DUTY_CYCLE => 0.500,
      CLKIN_PERIOD       => 50.0,
      REF_JITTER         => 0.016
   )
   port map
   (
      CLKFBOUT => pllout_clk_fb_dmtd,
      CLKOUT0  => pllout_clk_dmtd,
      CLKOUT1  => open,
      CLKOUT2  => open,
      CLKOUT3  => open,
      CLKOUT4  => open,
      CLKOUT5  => open,
      LOCKED   => open,
      RST      => '0',
      CLKFBIN  => pllout_clk_fb_dmtd,
      CLKIN    => clk_20m_vcxo_buf
   );

   -- L_RST_N is the reset line coming from Gennum linking dependencies
   local_reset_n <= not(EXTRA_SWITCH(1));

   cmp_clk_sys_buf : BUFG
   port map (
               O => clk_sys,
               I => pllout_clk_sys);

   cmp_clk_dmtd_buf : BUFG
   port map (
               O => clk_dmtd,
               I => pllout_clk_dmtd);

   cmp_clk_vcxo : BUFG
   port map (
               O => clk_20m_vcxo_buf,
               I => CLK20_VCXO);


   cmp_pllrefclk_buf : IBUFGDS
   generic map (
                  DIFF_TERM    => true,      -- Differential Termination
                  IBUF_LOW_PWR => true,      -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
                  IOSTANDARD   => "DEFAULT")
   port map (
               O  => s_fpga_clk,        -- Buffer output
               I  => FPGA_CLK_P,    -- Diff_p buffer input (connect directly to top-level port)
               IB => FPGA_CLK_N     -- Diff_n buffer input (connect directly to top-level port)
            );

  ------------------------------------------------------------------------------
  -- Dedicated clock for GTP
  ------------------------------------------------------------------------------
  cmp_gtp_dedicated_clk_buf : IBUFGDS
    generic map(
      DIFF_TERM    => true,
      IBUF_LOW_PWR => true,
      IOSTANDARD   => "DEFAULT")
    port map (
      O  => gtp_dedicated_clk,
      I  => FPGAMGTCLK0_P,
      IB => FPGAMGTCLK0_N
    );


   ------------------------------------------------------------------------------
   -- Active high reset
   ------------------------------------------------------------------------------
   s_rst <= EXTRA_SWITCH(1);

   wrc_slave_i.adr(16 downto 0)  <= wb_adr(16 downto 0);
   wrc_slave_i.adr(31 downto 17) <= (others => '0');

   process(clk_sys, s_rst)
   begin
      if rising_edge(clk_sys) then
         led_divider <= led_divider + 1;
      end if;
   end process;

   wrc_scl_i  <= '0';
   wrc_sda_i  <= '0';

   FPGA_SFP_SCL   <= '0' when sfp_scl_o = '0' else 'Z';
   FPGA_SFP_SDA   <= '0' when sfp_sda_o = '0' else 'Z';
   sfp_scl_i      <= FPGA_SFP_SCL;
   sfp_sda_i      <= FPGA_SFP_SDA;

   THERMOMETER <= '0' when owr_en(0) = '1' else 'Z';
   owr_i(0)  <= THERMOMETER;

   U_WR_CORE : xwr_core
   generic map
   (
      g_simulation                => 0,
      g_phys_uart                 => true,
      g_virtual_uart              => false,
      g_with_external_clock_input => true,
      g_aux_clks                  => 1,
      g_ep_rxbuf_size             => 1024,
      g_dpram_initf               => "",
      g_dpram_size                => 20480,
      g_interface_mode            => PIPELINED,
      g_address_granularity       => WORD
   )
   port map
   (
      clk_sys_i  => clk_sys,
      clk_dmtd_i => clk_dmtd,
      clk_ref_i  => s_fpga_clk,
      clk_aux_i  => (others => '0'),
      clk_ext_i  => dio_clk,
      pps_ext_i  => dio_in(3),
      rst_n_i    => local_reset_n,

      dac_hpll_load_p1_o => dac_hpll_load_p1,
      dac_hpll_data_o    => dac_hpll_data,
      dac_dpll_load_p1_o => dac_dpll_load_p1,
      dac_dpll_data_o    => dac_dpll_data,

      phy_ref_clk_i      => s_fpga_clk,
      phy_tx_data_o      => phy_tx_data,
      phy_tx_k_o         => phy_tx_k,
      phy_tx_disparity_i => phy_tx_disparity,
      phy_tx_enc_err_i   => phy_tx_enc_err,
      phy_rx_data_i      => phy_rx_data,
      phy_rx_rbclk_i     => phy_rx_rbclk,
      phy_rx_k_i         => phy_rx_k,
      phy_rx_enc_err_i   => phy_rx_enc_err,
      phy_rx_bitslide_i  => phy_rx_bitslide,
      phy_rst_o          => phy_rst,
      phy_loopen_o       => phy_loopen,

      led_red_o   => LED_WR_OK,
      led_green_o => LED_PPS,
      scl_o       => wrc_scl_o,
      scl_i       => wrc_scl_i,
      sda_o       => wrc_sda_o,
      sda_i       => wrc_sda_i,
      sfp_scl_o   => sfp_scl_o,
      sfp_scl_i   => sfp_scl_i,
      sfp_sda_o   => sfp_sda_o,
      sfp_sda_i   => sfp_sda_i,
      sfp_det_i   => FPGA_SFP_PRESENCE,
      btn1_i      => EXTRA_SWITCH(2),
      btn2_i      => EXTRA_SWITCH(3),

      uart_rxd_i => '0',
      uart_txd_o => open,

      owr_en_o => owr_en,
      owr_i    => owr_i,

      slave_i => wrc_slave_i,
      slave_o => wrc_slave_o,

      wrf_src_o => mbone_snk_in,
      wrf_src_i => mbone_snk_out,
      wrf_snk_o => mbone_src_in,
      wrf_snk_i => mbone_src_out,

      tm_dac_value_o       => open,
      tm_dac_wr_o          => open,
      tm_clk_aux_lock_en_i => '0',
      tm_clk_aux_locked_o  => open,
      tm_time_valid_o      => open,
      tm_utc_o             => open,
      tm_cycles_o          => open,
      pps_p_o              => pps,

      dio_o       => dio_out(4 downto 1),
      rst_aux_n_o => mbone_rst_n
   );


   -- Mini-BONE
   U_MiniBone : xmini_bone
   generic map
   (
      g_class_mask    => x"f0",
      g_our_ethertype => x"a0a0"
   )
   port map 
   (
      clk_sys_i => clk_sys,
      rst_n_i   => mbone_rst_n,
      src_o     => mbone_src_out,
      src_i     => mbone_src_in,
      snk_o     => mbone_snk_out,
      snk_i     => mbone_snk_in,
      master_o  => mbone_wb_out,
      master_i  => mbone_wb_in);

   U_DPRAM : xwb_dpram
   generic map
   (
      g_size                  => 2048,
      g_init_file             => "",
      g_must_have_init_file   => false,
      g_slave1_interface_mode => CLASSIC,
      g_slave2_interface_mode => CLASSIC,
      g_slave1_granularity    => WORD,
      g_slave2_granularity    => WORD
   )
   port map
   (
      clk_sys_i => clk_sys,
      rst_n_i   => mbone_rst_n,
      slave1_i  => mbone_wb_out,
      slave1_o  => mbone_wb_in,
      slave2_i  => dpram_slave2_in,
      slave2_o  => open
   );

   dpram_slave2_in.cyc <= '0';
   dpram_slave2_in.stb <= '0';


   U_GTP : wr_gtp_phy_spartan6
   generic map
   (
      g_simulation => 0
   )
   port map
   (
      gtp_clk_i          => gtp_dedicated_clk,

      ch0_ref_clk_i      => s_fpga_clk,
      ch0_tx_data_i      => phy_tx_data,
      ch0_tx_k_i         => phy_tx_k,
      ch0_tx_disparity_o => phy_tx_disparity,
      ch0_tx_enc_err_o   => phy_tx_enc_err,
      ch0_rx_data_o      => phy_rx_data,
      ch0_rx_rbclk_o     => phy_rx_rbclk,
      ch0_rx_k_o         => phy_rx_k,
      ch0_rx_enc_err_o   => phy_rx_enc_err,
      ch0_rx_bitslide_o  => phy_rx_bitslide,
      ch0_rst_i          => phy_rst,
      ch0_loopen_i       => phy_loopen,

      ch1_ref_clk_i      => s_fpga_clk,
      ch1_tx_data_i      => x"00",
      ch1_tx_k_i         => '0',
      ch1_tx_disparity_o => open,
      ch1_tx_enc_err_o   => open,
      ch1_rx_rbclk_o     => open,
      ch1_rx_data_o      => open,
      ch1_rx_k_o         => open,
      ch1_rx_enc_err_o   => open,
      ch1_rx_bitslide_o  => open,
      ch1_rst_i          => '1',
      ch1_loopen_i       => '0',

      pad_txn0_o         => MGTSFPTX0_N,
      pad_txp0_o         => MGTSFPTX0_P,
      pad_rxn0_i         => MGTSFPRX0_N,
      pad_rxp0_i         => MGTSFPRX0_P,
      pad_txn1_o         => open,
      pad_txp1_o         => open,
      pad_rxn1_i         => '0',
      pad_rxp1_i         => '0'
   );


   U_DAC_ARB : spec_serial_dac_arb
   generic map
   (
      g_invert_sclk    => false,
      g_num_extra_bits => 8
   )
   port map 
   (
      clk_i   => clk_sys,
      rst_n_i => local_reset_n,

      val1_i  => dac_dpll_data,
      load1_i => dac_dpll_load_p1,

      val2_i  => dac_hpll_data,
      load2_i => dac_hpll_load_p1,

      dac_cs_n_o(0) => FPGA_PLLDAC1_SYNC_N,
      dac_cs_n_o(1) => FPGA_PLLDAC2_SYNC_N,
      dac_clr_n_o   => open, --dac_clr1_n_o,
      dac_sclk_o    => s_dac_sclk_o,
      dac_din_o     => s_dac_din_o
   );

   FPGA_PLLDAC_SCLK <= s_dac_sclk_o;
   --dac_sclk2_o <= s_dac_sclk_o;
   FPGA_PLLDAC_DIN  <=  s_dac_din_o;
   --dac_din2_o  <=  s_dac_din_o;

   SFP_TX_DISABLE   <= '0';

end rtl;
